#include "FlareManager.h"

glm::vec2 CENTER_SCREEN(0.5, 0.5);

FlareManager::FlareManager(GLfloat spacing, Shader shader, GLuint screenW, GLuint screenH, FlareTexture flares[], GLfloat amount)
	: spacing(spacing), renderer(shader, screenW, screenH), amount(amount)
{
	this->init(flares);
}

void FlareManager::init(FlareTexture flares[]) {
	for (GLuint i = 0; i < amount; ++i)
		this->flares.push_back(flares[i]);
}

void FlareManager::render(glm::mat4 view, glm::mat4 projection, glm::vec3 lightPosition) {
	try {
		glm::vec2 lightCoords = convertToScreenSpace(lightPosition, view, projection);
		glm::vec2 sunToCenter(lightCoords.x - CENTER_SCREEN.x, lightCoords.y - CENTER_SCREEN.y);
	
		GLfloat brightness = 1 - (glm::length(sunToCenter) / 0.7f);
		if (brightness > 0) {
			calcFlarePositions(sunToCenter, lightCoords);
			renderer.Draw(flares, brightness/3);
		}
	}
	catch (std::invalid_argument) {
		return;
	}

}

void FlareManager::calcFlarePositions(glm::vec2 sunToCenter, glm::vec2 sunCoords) {
	for (GLfloat i = 0; i < amount; ++i) {
		glm::vec2 direction(i * sunToCenter);
		flares[i].setScreenPos(sunCoords.x + direction.x, sunCoords.y + direction.y);
	}
}

glm::vec2 FlareManager::convertToScreenSpace(glm::vec3 lightPosition, glm::mat4 view, glm::mat4 projection) {
	glm::vec4 coords(lightPosition, 1.0f);
	coords = view * coords;
	coords = projection * coords;
	if (coords.w <= 0) {
		throw std::invalid_argument("received null or negative w-value");
	}

	GLfloat x = (coords.x / coords.w + 1) / 2.0f;
	GLfloat y = 1 - ((coords.y / coords.w + 1) / 2.0f);
	return glm::vec2(x, y);
}