#include "FlareRenderer.h"

#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include <glm/gtc/type_ptr.hpp>

FlareRenderer::FlareRenderer(Shader shader, GLuint screenWidth, GLuint screenHeight)
	: shader(shader), screenWidth(screenWidth), screenHeight(screenHeight)
{
	this->init();
}

// Render all particles
void FlareRenderer::Draw(std::vector<FlareTexture> flares, GLfloat brightness)
{
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glDisable(GLU_CULLING);

	shader.Use();
	glUniform1f(glGetUniformLocation(shader.Program, "brightness"), brightness);

	for (FlareTexture flare : flares) {
		GLfloat xScale = flare.scale;
		GLfloat yScale = xScale * (GLfloat)screenWidth / screenHeight;
		
		glUniform4f(glGetUniformLocation(shader.Program, "transform"), flare.xPos-xScale/2, flare.yPos-yScale / 2, xScale, yScale);
		glActiveTexture(GL_TEXTURE0);
		glBindTexture(GL_TEXTURE_2D, flare.texture);

		glBindVertexArray(this->VAO);
		glDrawArrays(GL_TRIANGLES, 0, 6);
		glBindVertexArray(0);
	}

	glEnable(GL_DEPTH_TEST);
	glDisable(GL_BLEND);
	glEnable(GLU_CULLING);
}

void FlareRenderer::init()
{
	// Set up mesh and attribute properties
	GLuint VBO;
	GLfloat positions[] = {
		0.0f, 1.0f, 0.0f, 1.0f,
		1.0f, 0.0f, 1.0f, 0.0f,
		0.0f, 0.0f, 0.0f, 0.0f,

		0.0f, 1.0f, 0.0f, 1.0f,
		1.0f, 1.0f, 1.0f, 1.0f,
		1.0f, 0.0f, 1.0f, 0.0f
	};
	glGenVertexArrays(1, &this->VAO);
	glGenBuffers(1, &VBO);
	glBindVertexArray(this->VAO);
	// Fill mesh buffer
	glBindBuffer(GL_ARRAY_BUFFER, VBO);
	glBufferData(GL_ARRAY_BUFFER, sizeof(positions), positions, GL_STATIC_DRAW);
	// Set mesh attributes
	glEnableVertexAttribArray(0);
	glVertexAttribPointer(0, 4, GL_FLOAT, GL_FALSE, 4 * sizeof(GLfloat), (GLvoid*)0);
	glBindVertexArray(0);
}

