#pragma once
#include <glm/glm.hpp>
#include <glm/gtc/matrix_transform.hpp>
#include "Plane.h"
#include "Box.h"

namespace glm {
	class FrustumG {

	private:

		enum {
			TOP = 0, BOTTOM, LEFT,
			RIGHT, NEARP, FARP
		};
		#define ANG2RAD 3.14159265358979323846/180.0

	public:

		static enum { OUTSIDE, INTERSECT, INSIDE };
		Plane pl[6];
		vec3 ntl, ntr, nbl, nbr, ftl, ftr, fbl, fbr;
		float nearD, farD, ratio, angle, tang;
		float nw, nh, fw, fh;

		FrustumG::FrustumG() {
		}
		FrustumG::~FrustumG() {
		}

		void setCamInternals(float angle, float ratio, float nearD, float farD) {

			// store the information
			this->ratio = ratio;
			this->angle = angle;
			this->nearD = nearD;
			this->farD = farD;

			// compute width and height of the near and far plane sections
			tang = (float)tan(ANG2RAD * angle * 0.5);
			nh = nearD * tang;
			nw = nh * ratio;
			fh = farD  * tang;
			fw = fh * ratio;
		}		
		void setCamDef(vec3 &p, vec3 &l, vec3 &u) {

			vec3 dir, nc, fc, X, Y, Z;

			Z = p - l;
			normalize(Z);
			X = u * Z;
			normalize(X);
			Y = Z * X;			

			nc = p - Z * nearD;
			fc = p - Z * farD;

			// compute the 4 corners of the frustum on the near plane
			ntl = nc + Y * nh - X * nw;
			ntr = nc + Y * nh + X * nw;
			nbl = nc - Y * nh - X * nw;
			nbr = nc - Y * nh + X * nw;

			// compute the 4 corners of the frustum on the far plane
			ftl = fc + Y * fh - X * fw;
			ftr = fc + Y * fh + X * fw;
			fbl = fc - Y * fh - X * fw;
			fbr = fc - Y * fh + X * fw;

			// compute the six planes
			// the function set3Points assumes that the points
			// are given in counter clockwise order
			pl[TOP].set3Points(ntr, ntl, ftl);
			pl[BOTTOM].set3Points(nbl, nbr, fbr);
			pl[LEFT].set3Points(ntl, nbl, fbl);
			pl[RIGHT].set3Points(nbr, ntr, fbr);
			pl[NEARP].set3Points(ntl, ntr, nbr);
			pl[FARP].set3Points(ftr, ftl, fbl);
		}		

		int pointInFrustum(vec3 &p) {
			int result = INSIDE;
			for (int i = 0; i < 6; i++) {
				if (pl[i].distance(p) < 0)
					return OUTSIDE;
			}
			return(result);
		}	

		int boxInFrustum(Box &b) {
			int result = INSIDE;
			for (int i = 0; i < 6; i++) {
				if (pl[i].distance(b.getVertexP(pl[i].normal)) < 0)
					return OUTSIDE;
				else if (pl[i].distance(b.getVertexN(pl[i].normal)) < 0)
					result = INTERSECT;
			}
			return(result);
		}
	};
}






