#ifndef PARTICLE_GENERATOR_H
#define PARTICLE_GENERATOR_H
#include <vector>

#include <GL/glew.h>
#include <glm/glm.hpp>

#include "shader.h"
//#include "texture.h"
//#include "game_object.h"


// Represents a single particle and its state
struct Particle {
	glm::vec3 Position, Velocity;
	GLfloat Rotation, Scale, Life;

	Particle() : Position(0.0f), Velocity(0.0f), Rotation(0.0f), Scale(1.0f), Life(0.0f) { }
};


// ParticleGenerator acts as a container for rendering a large number of 
// particles by repeatedly spawning and updating particles and killing 
// them after a given amount of time.
class ParticleGenerator
{
public:
	// Constructor
	ParticleGenerator(Shader shader, GLuint amount);
	// Update all particles
	void Update(GLfloat dt, glm::vec3 position, GLuint newParticles, glm::vec3 offset = glm::vec3(0.0f, 0.0f, 0.0f), GLboolean collected = false);
	// Render all particles
	void Draw(glm::mat4 &projection, glm::mat4 &view);
private:
	// State
	std::vector<Particle> particles;
	GLuint amount;
	// Render state
	Shader shader;
	//Texture2D texture;
	GLuint VAO;
	// Initializes buffer and vertex attributes
	void init();
	// Returns the first Particle index that's currently unused e.g. Life <= 0.0f or 0 if no particle is currently inactive
	GLuint firstUnusedParticle();
	// Respawns particle
	void respawnParticle(Particle &particle, glm::vec3 position, glm::vec3 offset = glm::vec3(0.0f, 0.0f, 0.0f));
	void ParticleGenerator::UpdateModelViewMatrix(glm::vec3 position, float rotation, float scale, glm::mat4 view);
	GLfloat RandomFloat(GLfloat a, GLfloat b);
};

#endif

