#version 330 core
in vec3 FragPos;
in vec3 Normal;
//in vec4 vertexColor; // The input variable from the vertex shader (same name and same type)
  
out vec4 color;

uniform vec4 objectColor;
uniform vec3 lightPos;
uniform vec3 lightColor;
uniform vec3 viewPos;

void main()
{

	//ambient
    float ambientStrength = 0.1f;
    vec3 ambient = ambientStrength * lightColor;

	//diffuse
	vec3 norm = normalize(Normal);
	vec3 lightDir = normalize(lightPos - FragPos); 

	float diff = max(dot(norm, lightDir), 0.0);
	vec3 diffuse = diff * lightColor;

	//specular
	float specularStrength = 0.8f;
	vec3 viewDir = normalize(viewPos - FragPos);
	vec3 reflectDir = reflect(-lightDir, norm);

	float spec = pow(max(dot(viewDir, reflectDir), 0.0), 32);
	vec3 specular = specularStrength * spec * lightColor; 


	vec4 result = vec4((ambient + diffuse + specular), 1.0f) * objectColor;
	color = vec4(result.x, result.y, result.z, 1.0f);
} 